/* *******************************************************************
   ESP8266 Neopixel Testsketch

   noiasca: 

Short
   Test Sketch for RGB Leds Neopixel/WS2811/WS2812/APA106 

   *******************************************************************
   
Software
- AJAX    update website via Ajax
- EEPROM  Save Settings to EEPROM (improved Version with less commits)
- OTA     Over the Air firmware update from a webserver resource to ESP (fixed webserver)

Libraries & Credits
- Adafruit Neopixel library

Hardware
- Neopixel RGB LEDS like Neopixel/WS2811/WS2812/APA106 

Version 
- HTML5 with tables
- Single WIFI

OPEN TASKS
- Multiwifi
- Durch Tastendruck Szene/Pattern aktivieren für eine Zeit N
- Durchlaufzeit N parametrierbar machen
- Schnellzugriff und Manuell trennen
- Kanäle zum Bennenen (und dann durchziehen für die Sliderbeschriftung in "Manuel"
- Helligkeitsregelung (Wann Dunkel Szene/Pattern X laden)
- Programm bei Dunkelheit / Helligkeit (das erstezt dann auch den Effekt nach Einschalten)
- Dunkelheit/Helligkeit erst nach Idle von x Sekunden einschalten
- Bei Tastendruck für x Sekunden Szene/Pattern laden

***************************************************************** */

#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>
#include <ESP8266mDNS.h>
#include <EEPROM.h>
#include <ESP8266httpUpdate.h>                        // added support for OTA Updates
#include "rgb.h"

#define USE_BOARD 101                                  // the actual board to compile


/* ******************************************************************* 
 *       the board settings / die Einstellungen der verschiedenen Boards
 ********************************************************************/
 
#define VERSION "12.1"                                // This is just the version of the Arduino Sketch
#define TXT_BOARDNAME "ESP8266 Neopixel Test"         // The name of the board

#if USE_BOARD == 101                                   // Example Board
  #define TXT_BOARDID "101"                            // an ID for the board : 101 as an example
  const char* ssid = "your SSID here";                 // set WiFi SSID
  const char* password = "your WLAN passphrase here";  // set WiFi password
#endif



/* ******************************************************************* 
 *       other settings / weitere Einstellungen für den Anwender
 ********************************************************************/

#define HTTP_EXTERNAL_CSS       "<link rel='stylesheet' href='http://your.web server.net/css/external.css'>" //set path to your webserver with the external.css
#define HTTP_UPDATE_SERVERNAME  "http://your.web server.net"        // Server where you will upload OTA Updates
#define HTTP_UPDATE_SERVERPATH  "/update"                           // start with a backslash, the final path will be this HTTP_UPDATE_SERVERPATH + "/" + TXT_BOARDID+"/" + HTTP_UPDATE_BINNAME
#define HTTP_UPDATE_BINNAME     "esp8266_neopixel.nodemcu.bin"      // Arduino IDE generates following: folder_name_of_the_skectch + your_ESP_Board_family + .bin        

#include "web_en.h"           // English website - one textfile must be activated 
//#include "web_de.h"             // German website

/* ******************************************************************* 
 *       Globals - Variables and constants
 ********************************************************************/
 
const char* colors ="wrgb";                           // just to have the letters for colors in an array for easier use in Loops 

#ifdef __AVR__
  #include <avr/power.h>
#endif

//PIN definitions
#define LED_RED_PIN       16                      // the red LED on the NodeMCU 1.0 is GPIO16/D0
//#define LED_BLUE_PIN    02                      // the blue LED on the ESP-12e is GPIO02/D4 - same like Serial2
#define RGB_PIN           D2                      // WS2811. //geht am ESP: D1 D2 D4 10    bei D4 leuchtet die blaue LED am ESP permanent // geht nicht am ESP: D0    

//Variables and constants
ESP8266WebServer server(80);                      // Port of ESP webserver
uint32_t maxChannels = 51;                        // How many Channels are used by the USER before EEPROM is readed
const unsigned int maxChannelsTechnical = 255;    // should supports 512 channels but as this sketch uses a lot of byte variables - we are limited to 255. 
byte master = 100;                                // Mastervolume
byte channel [maxChannelsTechnical];              // the actual value of each channel

byte r_channel [maxChannelsTechnical];            // the actual value of each channel
byte g_channel [maxChannelsTechnical];            // the actual value of each channel
byte b_channel [maxChannelsTechnical];            // the actual value of each channel

byte rgbState;                // stores the actual State of the RGB LED - MISSING nur zum debuggen

//byte sceneChannel[255];                         // a scene can store a maximum auf 255 values. here we store the channel - was replaced by eeprom
//byte sceneValue[255];                           // a scene can store a maximum auf 255 values. here we store the value itself - was replaced by eeprom
                            
unsigned int ajaxIntervall = 60;                  // Intervall for AJAX call of website in seconds, updateable via webserver
const byte ajaxIntervallMin = 5;                  // Minimum for ajaxIntervall in seconds.
boolean useAjax = true;                           // should the website refreshed with AJAX, updateable via webserver
                                                  // MISSING: get rid of the magical number 11 for channel data ...
boolean isBlackout = false;                       // Blackout
unsigned long ss = 0;                             // actual second
//unsigned long ssTimerMinute = 0;                // Timer variable to store old value for Minute Timer
//unsigned long ssTimerHour = 0;                  // Timer variable to store old value for Hour Timer
unsigned long msTimerPixel = 0;                   // Timer variable to store Pixel call
unsigned int msTimerPixelIntervall = 1000; 
unsigned long msTimerPattern = 0;                 // Timer variable to store Pattern call
unsigned int msTimerPatternIntervall = 100;       // Intervall for Pattern (i.e. Fade pattern)
const byte patternIntervallFactor = 50;           // Pattern Intervall will be multiplied by this factor ... 50 x 255 gives an intervall from 0,050 to 12,75 seconds
unsigned long msTimerIdle = 0;                    // Timer variable to Idle call
unsigned int msTimerIdleIntervall = 1000; 

byte actualPattern = 0;                           // Which pattern is currently activ
unsigned int counterPattern = 0;                  // For Pattern which need a global counter 
const byte maxRows = 20;                          // how many rows should be displayed per page
unsigned int sceneCache = maxRows+1;              // caches the count of scenes
const byte maxPattern = 7;                        // how many Patterns are available
boolean useLocalCss = false;                      // is a local.css loaded to the SPIFFS 

// EEPROM Adresses
//const byte addrWlan = 0;                        // some sketches us this address to store a wlan-Index. currently not used in this sketch
const byte addrMaxChannels = 1;                   
// reserve address 2 for addrMaxChannels for migration to integer
const byte addrUseAjax = 6;
const byte addrAjaxIntervall = 7;                 // uses byte 7 and 8
const byte addrPixelIntervall = 9;                // uses byte 9 and 10
const byte addrPatternIntervall = 11;             // 254 ... 25.4 seconds (eigentlich nur Byte aber resever auf integer)
const byte addrNightId = 13;                      // Scene / Pattern to load at night     (0..x Scene / x..254 Pattern / 255 nothing)
const byte addrDayId = 15;                        // Scene / Pattern to load during day
const byte addrIdleIntervall = 17;                // 
const byte addrChannels = 100;                    // Start adress. 0 Capabilities | 1..8 Memo  | 9..10 reserved = 11 bytes. 255*11=2805
                                                  // 4 byte buffer... (channels end at 100+255*11=2905
const unsigned int addrScene = 2910;              // 0 length || 1..8 Memo |[ 9..channel | 10 r_value | 11 g_value | 12 b_value ]       max useable space is up to addrSequence-1
const unsigned int addrSequence = 3700;

#include <Adafruit_NeoPixel.h>                    // WS2811/WS2812/APA10 Library
//Adafruit_NeoPixel strip = Adafruit_NeoPixel(maxChannels, RGB_PIN, NEO_GRB + NEO_KHZ400);  //WS2812 Breakout: GRB
Adafruit_NeoPixel strip = Adafruit_NeoPixel(maxChannels, RGB_PIN, NEO_RGB + NEO_KHZ800);    //APA106: RGB. Testweise 800


/* ******************************************************************* 
 *       S E T U P
 ********************************************************************/

void setup(void){
  //PIN initialize
  pinMode(LED_RED_PIN, OUTPUT);
  digitalWrite(LED_RED_PIN, HIGH);

  Serial.begin(115200);
  Serial.println(F(""));
  Serial.println(F(TXT_BOARDNAME));
  Serial.println(F("Version: " VERSION " Board " TXT_BOARDID));

  // Read stored values from EEPROM
  EEPROM.begin(4096);  
  useAjax=EEPROM.read(addrUseAjax);
  ajaxIntervall=constrain(EEPROMReadinteger(addrAjaxIntervall),ajaxIntervallMin,32767);            //das sind über 5 Stunden - das soll reichen.
  maxChannels=constrain(EEPROMReadinteger(addrMaxChannels),1,maxChannelsTechnical);  
  msTimerPixelIntervall=constrain(EEPROMReadinteger(addrPixelIntervall),100,32767);    
  msTimerPatternIntervall=constrain(EEPROM.read(addrPatternIntervall),1,255);  
  
  Serial.println(F("I: Init RGB Pixel"));

  strip.begin();
  strip.show();
  memset(r_channel,0,sizeof(r_channel));
  memset(g_channel,0,sizeof(g_channel));
  memset(b_channel,0,sizeof(b_channel));
  
  WiFi.begin(ssid, password);

  // Wait for connection
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(F("."));
  }
  Serial.print(F("\nConnected to "));
  Serial.println(ssid);
  Serial.print(F("IP address: "));
  Serial.println(WiFi.localIP());

  if (MDNS.begin("esp8266")) {
    Serial.println(F("MDNS responder started"));
  }

  server.on("/",      handlePage);
  server.on("/0.htm", handlePage);     // most Pages have the same header/footer and share the same function - the function cares about the different sizes by URI
  server.on("/1.htm", handlePage);     // Channel Overview
  server.on("/2.htm", handlePage);     // Settings
  server.on("/3.htm", handlePage);     // Channel Detail
  server.on("/4.htm", handlePage);     // Scene Overview
  server.on("/5.htm", handlePage);     // Scene Detail
  server.on("/f.css", handleCss);      // stylesheet
  server.on("/.js",      handleJs);    // JavaScript differs only in small parts (i.e. fields to be updated and used functions)
  server.on("/0.htm.js", handleJs);
  server.on("/1.htm.js", handleJs);
  server.on("/2.htm.js", handleJs);
  server.on("/3.htm.js", handleJs);
  server.on("/4.htm.js", handleJs);
  server.on("/5.htm.js", handleJs);
  server.on("/ajax", handleAjax);      // with some modifcation via paramter 

  //server.on("/", handleRoot);
  //server.on("/inline", [](){
  //  server.send(200, "text/plain", "this works as well");
  //});

  server.onNotFound(handleNotFound);    // cool for a typical 404 page
  server.begin();
  Serial.println(F("HTTP server started"));
}

/* ******************************************************************* 
 *       M A I N L O O P
 ********************************************************************/
 
void loop(void){
  ss=millis()/1000;
  server.handleClient();
 
  if ((unsigned long)(millis() - msTimerPixel) >= msTimerPixelIntervall)        
  {    
    //Serial.println(F("D226: TimerPixel"));
    //Serial.println(analogRead(A0));             //MISSING: Just Temp Debug Information of LDR Sensor
    updatePixel();
    msTimerPixel=millis();
  }
  
  if (!isBlackout && actualPattern > 0 && (unsigned long)(millis() - msTimerPattern) >= (msTimerPatternIntervall*patternIntervallFactor))        
  {    
    if (actualPattern == 1) {fadePixel();}        //bei 5 Pattern ist ein if else um etwa 16 byte kleiner als ein switch case
    else if (actualPattern == 2) {pattern2();}
    else if (actualPattern == 3) {pattern3();}
    else if (actualPattern == 4) {pattern4();}
    else if (actualPattern == 5) {pattern5();}
    else if (actualPattern == 6) {pattern6();}
    else if (actualPattern == 7) {pattern7();}    
    msTimerPattern=millis();
  }
   
  if (!isBlackout && actualPattern == 0) 
  {
                                                      //  MISSING: Tag/Nacht Unterscheidung
        byte current = EEPROM.read(addrNightId);
        //Serial.print(F("D244 current:"));Serial.println(current);
        if (current<255 && current>=255-maxPattern) 
        { 
          actualPattern = 255-current;                // MISSING irgendwie erwischen wir noch nicht alle pattern 1-5
          if (actualPattern==1)                       // before we switchover to fade wie set colors to random
          {
            for (int j = 0; j<maxChannels; j++) 
            {
              r_channel[j] = random(0,255);
              g_channel[j] = random(0,255);
              b_channel[j] = random(0,63);  // blue is very intensive ... limited to lower value
            }
          }
        }
        else if (current<255-maxPattern) 
        {loadScene(current);}  
  }  
  
}

/* ******************************************************************* 
 *       F U N C T I O N S
 ********************************************************************/






 
