 /* *******************************************************************
   Webserver

   How it all works together:
   
   page   0.htm         includes javascript 0.htm.js
   script 0.htm.js      differs for each page (fields), and calls ajax?p=/0.htm.js 
          ajax          differs for the pages and expects the called page as parameter p and returns data as comma separated file                        
   css    f.css         one css for all included by the pages
          external.css  one (complex) css on an external webserver. Additional 3,27 KB

   2017-01-07 Migration to HTML5
      
   ***************************************************************** */

void handleRoot() 
{
  digitalWrite(LED_RED_PIN, 1);
  server.send(200, "text/plain", "hello from esp8266!");
  digitalWrite(LED_RED_PIN, 0);
}


void handleNotFound()
{
  digitalWrite(LED_RED_PIN, 1);
  String message = "404 - File Not Found\n\n";
  message += "URI: ";                                //ohne F macro RAM 265.847 GV 40.300 mit F macro 265.935 GV 40.284
  message += server.uri();
  message += "\nMethod: ";
  message += (server.method() == HTTP_GET)?"GET":"POST";
  message += "\nArguments: ";
  message += server.args();
  message += "\n";
  for (uint8_t i=0; i<server.args(); i++){
    message += " " + server.argName(i) + ": " + server.arg(i) + "\n";
  }
  server.send(404, "text/plain", message);
  digitalWrite(LED_RED_PIN, 0);
}


void handlePage()
{
  digitalWrite(LED_RED_PIN, 1);
  Serial.print (F("D33  requested page ")); Serial.println(server.uri());
  if (server.args()>0)
  {    
    Serial.print(F("     Parameter "));
    Serial.println(server.args());
    for (uint8_t i=0; i<server.args(); i++)
    {
      Serial.print (F("     "));
      Serial.print (server.argName(i));
      Serial.print (F(": "));
      Serial.println(server.arg(i));;
    }
  }
  String message;
  message =F("<!doctype html>\n" 
             "<html>\n<head>\n<title>" TXT_BOARDNAME " - Board " TXT_BOARDID "</title>\n"
             "<meta charset='utf-8'>\n" 
             "<link rel='stylesheet' href='/f.css'>\n");                                            // 1. a simple Stylesheet generated by the CPU
  if (useLocalCss)
  {
    message +=F("<link rel='stylesheet' href='/i.css'>\n");                                         // 2. a complex Stylesheet in the SPIFFS
  }  
  message +=F( HTTP_EXTERNAL_CSS "\n"                                                               // 3. a complex Stylesheet on an external Server 
             "<meta name=\"viewport\" content=\"width=device-width\">\n");
  if (useAjax || server.uri()=="/0.htm" || server.uri()=="/" || server.uri()=="/3.htm" || server.uri()=="/4.htm")
  {
     message +=F("<script src='");
     message += server.uri();
     message +=F(".js'></script>\n");
  }
  message +=F("</head>\n");
  if (useAjax)
    {message +=F("<body onload='GetSwitchState(0)'>\n");}
  else
    {message +=F("<body>\n");}
  message +=F("<header>\n<h1>" TXT_BOARDNAME " - Board " TXT_BOARDID "</h1>\n"
             "<nav><p><a href='0.htm'>" TXT_HOME "</a> <a href='4.htm'>" TXT_SCENES "</a> <a href='1.htm'>" TXT_CHANNELS "</a> <a href='2.htm'>" TXT_SETTINGS "</a></p></nav></header>\n"
             "<main>\n");
  //message += server.uri();
  
  if (server.uri()=="/0.htm" || server.uri()=="/")                            //*** HOME ***
  {
    //strip.show();     // nur falls die Ausgabe länger dauern sollte ... MISSING wirkt aber nicht wie gewollt
    unsigned int start=0;
    if (server.argName(0)=="start")
    {
      start=server.arg(0).toInt();                     
    }
    if (start<0) start=0;

    message +=F("<article class='pgmc'>"     
                "<h2>" TXT_QUICK "</h2>\n");

    message +=F("<button type='button' value='255' id='ablack' onclick='fire(this.id, 255)'>" TXT_BLACKOUT "</button> \n"
                "<button type='button' value='255' id='asceneload' onclick='fire(this.id, 0)'>" TXT_LOAD " " TXT_SCENE " 0</button> \n"
                "<button type='button' value='255' id='ascenesave' onclick='fire(this.id, 0)'>" TXT_SAVE " " TXT_SCENE " 0</button> \n"
                "<button type='button' value='255' id='arnd' onclick='fire(this.id, 255)'>" TXT_RANDOM "</button> \n"
                "<button type='button' value='255' id='adel' onclick='fire(this.id, 255)'>" TXT_DELETE "</button> \n"
                "<button type='button' value='255' id='afade' onclick='fire(this.id, 255)'>" TXT_FADE "</button> \n"
                "<button type='button' value='255' id='apattern2' onclick='fire(this.id, 255)'>" TXT_PATTERN2 "</button>\n"
                "<button type='button' value='255' id='apattern3' onclick='fire(this.id, 255)'>" TXT_PATTERN3 "</button>\n"
                "<button type='button' value='255' id='apattern4' onclick='fire(this.id, 255)'>" TXT_PATTERN4 "</button>\n"
                "<button type='button' value='255' id='apattern5' onclick='fire(this.id, 255)'>" TXT_PATTERN5 "</button>\n"  
                "<button type='button' value='255' id='apattern6' onclick='fire(this.id, 255)'>" TXT_PATTERN6 "</button>\n"
                "<button type='button' value='255' id='apattern7' onclick='fire(this.id, 255)'>" TXT_PATTERN7 "</button>\n"  
                "<br><br>\n");

    message +=F("<h2>" TXT_MANUAL "</h2>\n");
    if (maxChannels>maxRows)
    {
      message+=F("<div id='subm'>");                      // submenu
      byte pages = maxChannels/maxRows+1;                    // make a small menubar to navigate through the scenes
      for (unsigned int i = 0; i<pages; i++)              
      {
        message+=F(" <a href='?start=");
        message+=i*maxRows+0;   
        message+=F("'");
        if (start>=i*maxRows && start<((i+0)*maxRows)) message+=F(" class='curr'");    //current menu entry fits to this page
        message+=F(">");
        message+=i*maxRows+0;
        message+=F("..</a>");
      }
      message+=F("</div>");
    }

    message +=F("<table>\n<tr><td></td>\n<td>" TXT_MASTER "</td>\n<td><input class='col3' type='range' step='1' min='0' max='100' value='"); 
    message += master;
    message +=F("' id='master' onChange='fire(this.id, this.value)'/></td></tr>\n");
 
    byte rgb=0;                                                             // 0 no RGB | 1 white | 2 red  | 3 green | 4 blue  
    //for (int j = 0; j<constrain(maxChannels,1,25); j++)                      // The sliders for each PIXEL LED  //MISSING - has problems with large tables ... therefore constrain to 25
    
    for (int j=start; j<start+maxRows && j<maxChannels; j++)                 // The sliders for each PIXEL LED
    { 
      
      byte helper[4];           //for easier use in loop
      helper[0]=255;
      helper[1]=r_channel[j];
      helper[2]=g_channel[j];
      helper[3]=b_channel[j];
      for (int i = 0; i<4; i++)                              
      {
        message +=F("<tr>\n<td>");
        message +=j;
        message +=F("</td>\n<td>");
        //message +=EEPROMReadstring(addrChannels+1+j*11);
        message +=F("</td>\n<td class='bgc");
        message +=i+1;
        message +=F("'><input class='col3 ran");
        message +=i+1;
        message +=F("' type='range' step='1' min='0' max='255' value='"); 
        message += helper[i];
        message +=F("' id='");
        message +=colors[i]; //just to get w r g or b as id-trailer
        message +=j;
        message +=F("' onChange='fire(this.id, this.value)'/></td></tr>\n");
      }
      

      
    }
    message +=F("</table><iframe name='i' style='display:none'></iframe>\n"
                "</article>\n");
  
  } //End of page 0

  else if (server.uri()=="/1.htm")                                                  //*** 1 Channels Overview ***
  {
    //first we care for the parameters 
    unsigned int start=0;
    if (server.argName(0)=="start")
    {
      start=server.arg(0).toInt();                     
    }
    //if (start<1) start=1;
    // now the HTML   
    
    message +=F("<article class='pgmc'>\n"
              "<h2>" TXT_CHANNEL " " TXT_OVERVIEW "</h2>\n");
    if (maxChannels>maxRows)
    {
      message+=F("<div id='subm'>");                       // submenu
      byte pages = maxChannels/maxRows;                    // make a small menubar to navigate through the scenes
      for (unsigned int i = 0; i<=pages; i++)              
      {
        message+=F(" <a href='?start=");
        message+=i*maxRows+1;   
        message+=F("'");
        if (start>=i*maxRows && start<((i+1)*maxRows)) message+=F(" class='curr'");    //current menu entry fits to this page
        message+=F(">");
        message+=i*maxRows+1;
        message+=F("..</a>");
      }
      message+=F("</div>");
    }
    message+=F("<table>\n<tr><td>" TXT_NR "</td>"
               "<td colspan='8'>" TXT_CAPABILITY "</td>"
               "<td></td>"
               "<td>" TXT_MEMO "</td>"
               "<td></td></tr>\n");

    //const char *txtCapability[] = {TXT_SLIDER,TXT_PUSH,"",TXT_RGBGROUP,"","","",""};     //MISSING: anders gelöst. mit 3 If weniger RAM. Könnte gelöscht werden.         
    for (int i=start; i<start+maxRows && i<=maxChannels; i++) 
    {
      message+=F("<tr>\n<td class='col'>");
      message+=i;
      message+=F("</td>\n");
      for (int j= 0; j<8; j++)
      {
        if (EEPROM.read(addrChannels+i*11) & (1<<j))
        {
          message+=F("<td title='");
          if (j==0) message+=TXT_SLIDER;
          else if (j==1) message+=TXT_PUSH;
          else if (j==3) message+=TXT_RGBGROUP;
          message+=F("'>");
          message+=F("&#x2718;");  // "checked"   &#x2718; or &#9745; or &#9746; or  http://www.w3schools.com/charsets/ref_utf_symbols.asp
        }
        else
        {
          message+=F("<td>");
          message+=F("&#x25A1;");  // "unchecked" &#x25A1; or &#9744; or            
        }
        message+=F("</td>\n");
      }
      message +=F("<td> = ");
      message+=EEPROM.read(addrChannels+i*11);
      message +=F(" </td>\n<td>");      //3
      message+=EEPROMReadstring(addrChannels+1+i*11);
      message+=F("</td>\n"
                 "<td><a href='3.htm?channel=");
      message+=i;           
      message+=F("'>" TXT_EDIT "</a></td></tr>\n");
    }
    message +=F("</table>\n</article>\n");                         //pgmc

  } //end of page 1 specifica
  
  else if (server.uri()=="/2.htm")                                                  //*** 2 - Settings - Einstellungen ***
  {
   // we start with the parameter handling for this page

    if (server.argName(0)=="otaUpdate") 
    {
       Serial.println(F("D229 OTA Update will need aprox. 45 Seconds ..."));








       t_httpUpdate_return ret = ESPhttpUpdate.update(HTTP_UPDATE_SERVERNAME HTTP_UPDATE_SERVERPATH "/"  + server.arg(0));
       switch(ret) 
       {
          case HTTP_UPDATE_FAILED:
            Serial.printf("E: HTTP_UPDATE_FAILD Error (%d): %s\n", ESPhttpUpdate.getLastError(), ESPhttpUpdate.getLastErrorString().c_str());
            break;
          case HTTP_UPDATE_NO_UPDATES:
            Serial.println(F("E: HTTP_UPDATE_NO_UPDATES"));
            break;
          case HTTP_UPDATE_OK:
            Serial.println(F("E: HTTP_UPDATE_OK"));
            break;
        }     
    }

    if (server.argName(0)=="ajaxIntervall") 
    {
      Serial.println(F("D264 Parameter addrAjaxIntervall"));
      ajaxIntervall=constrain(server.arg(0).toInt(),ajaxIntervallMin,32767);
      EEPROMWriteinteger(addrAjaxIntervall, ajaxIntervall);
      if (server.args()==2)
      { 
        //Serial.println(F("D269 set"));
        useAjax=true; 
        EEPROM.write(addrUseAjax,1);
        EEPROM.commit();
      }
      else
      {
        //Serial.println(F("D274 delete"));
        useAjax=false;
        EEPROM.write(addrUseAjax,0);
        EEPROM.commit();
      }
    }
    else if (server.argName(0)=="pixelIntervall") 
    {
      Serial.println(F("D187 Parameter pixelIntervall"));
      msTimerPixelIntervall=constrain(server.arg(0).toInt(),ajaxIntervallMin,32767);
      EEPROMWriteinteger(addrPixelIntervall, msTimerPixelIntervall);
    }
    else if (server.argName(0)=="patternIntervall") 
    {
      Serial.println(F("D225 Parameter fadeIntervall"));
      msTimerPatternIntervall=constrain(server.arg(0).toInt(),1,255);
      EEPROM.write(addrPatternIntervall, msTimerPatternIntervall);
      EEPROM.commit();
    }
    else if (server.argName(0)=="nightId") 
    {
      Serial.println(F("D300 Parameter nightId"));
      EEPROM.write(addrNightId, constrain(server.arg(0).toInt(),0,255));
      EEPROM.commit();
    } 
    else if (server.argName(0)=="dayId") 
    {
      Serial.println(F("D306 Parameter dayId"));
      EEPROM.write(addrDayId, constrain(server.arg(0).toInt(),0,255));
      EEPROM.commit();
    }        
    else if (server.argName(0)=="maxChannels") 
    {
      //Serial.println(F("D227 Paramter maxChannels"));
      int oldMaxChannels = maxChannels;
      maxChannels=constrain(server.arg(0).toInt(),1,maxChannelsTechnical);
      EEPROMWriteinteger(addrMaxChannels,maxChannels);
      if (maxChannels < oldMaxChannels)                             // Reset old channels
      {
        for (int j = maxChannels+1; j<=oldMaxChannels; j++)              
        {
          r_channel[j]=0;g_channel[j]=0;b_channel[j]=0;
         }
        updatePixel();
      }
    }
    
    // now the html
    
    message +=F("<article class='pgmc'>\n"    
                "<h2>" TXT_SETTINGS "</h2>\n" 
                "<table>\n" 
                "<tr><form action='#' method='GET'><td class='col3' >" TXT_COUNT " " TXT_CHANNELS ":</td>" 
                "<td><input class='col2' name='maxChannels' value='");   
    message += maxChannels;
    message +=F("'></td><td></td><td><input class='col2' type='submit' value='" TXT_SAVE "'></form></tr>\n"

                "<tr><form action='#' method='GET'><td class='col3' >Pixel " TXT_INTERVALL " (ms):</td>" 
                "<td><input class='col2' name='pixelIntervall' value='");   
    message += msTimerPixelIntervall;
    message +=F("'></td><td></td><td><input class='col2' type='submit' value='" TXT_SAVE"'></form></tr>\n"

                   "<tr><form action='#' method='GET'><td class='col3' >" TXT_PATTERN" " TXT_INTERVALL" (*");           // patternIntervall
    message += patternIntervallFactor;            
    message +=F("ms):</td>"                
                "<td><input class='col2' name='patternIntervall' value='");   
    message += msTimerPatternIntervall;
    message +=F("'></td><td></td><td><input class='col2' type='submit' value='" TXT_SAVE "'></form></tr>\n"
    
                "<tr><form action='#' method='GET'><td class='col3' >" TXT_UPDATE":"                                   // OTA Updater
                "</td><td><input class='col2' name='otaUpdate' value='");   
    message += TXT_BOARDID "/" HTTP_UPDATE_BINNAME;                                                                     // only boardid and bin File can be choosen!!!
    message +=F("'></td><td></td><td><input class='col2' type='submit' value='" TXT_START "'></form></tr>\n"
   
                "<tr><form action='#' method='GET'><td class='col3' >" TXT_AJAX " " TXT_INTERVALL " (s):</td>"        // ajaxIntervall
                "<td><input class='col2' name='ajaxIntervall'  value='");                                             
    message += ajaxIntervall;
    message +=F("'>"
                "</td><td><input class='col' type='checkbox' name='useAjax' value='checked' id='useAjax' ");
    if (useAjax) {message +=" checked='checked'";}
    message +=F("></td><td><input class='col2' type='submit' value='" TXT_SAVE "'></form></tr>\n"

                "<tr><form action='#' method='GET'><td class='col3' >" TXT_NIGHT_ID ":"                                   // Scene or Pattern to load at Night
                "</td><td><select class='col2' name='nightId'>\n");
    for (unsigned int i=0; i < sceneCache; i++)
    {
       unsigned int sceneAddr = getSceneAddr(i);
       if (EEPROMReadstring(sceneAddr+1)!="")
       {
         message +=F(" <option value='");
         message +=i;
         message +=F("'");
         if (EEPROM.read(addrNightId)==i) message+=F(" selected");
         message +=F(">");
         message +=EEPROMReadstring(sceneAddr+1);     // 4 Memo
         message +=F("</option>\n");
       }
    }
         message +=F(" <option value='248'"); if(EEPROM.read(addrNightId)==248) message+=F(" selected");message +=F(">" TXT_PATTERN7 "</option>\n");
         message +=F(" <option value='249'"); if(EEPROM.read(addrNightId)==249) message+=F(" selected");message +=F(">" TXT_PATTERN6 "</option>\n");
         message +=F(" <option value='250'"); if(EEPROM.read(addrNightId)==250) message+=F(" selected");message +=F(">" TXT_PATTERN5 "</option>\n");
         message +=F(" <option value='251'"); if(EEPROM.read(addrNightId)==251) message+=F(" selected");message +=F(">" TXT_PATTERN4 "</option>\n");
         message +=F(" <option value='252'"); if(EEPROM.read(addrNightId)==252) message+=F(" selected");message +=F(">" TXT_PATTERN3 "</option>\n");
         message +=F(" <option value='253'"); if(EEPROM.read(addrNightId)==253) message+=F(" selected");message +=F(">" TXT_PATTERN2 "</option>\n");
         message +=F(" <option value='254'"); if(EEPROM.read(addrNightId)==254) message+=F(" selected");message +=F(">" TXT_FADE "</option>\n");
         message +=F(" <option value='255'"); if(EEPROM.read(addrNightId)==255) message+=F(" selected");message +=F(">" TXT_NONE "</option>\n");
   message +=F("</select>\n</td><td></td><td><input class='col2' type='submit' value='" TXT_SAVE "'></form></tr>\n"



/*                
                "<tr><form action='#' method='GET'><td class='col3' >" TXT_NIGHT_ID ":"                                   // Scene or Pattern to load at Night
                "</td><td><input class='col2' name='nightId' value='");   
    message += EEPROM.read(addrNightId);
    message +=F("'></td><td></td><td><input class='col2' type='submit' value='" TXT_SAVE "'></form></tr>\n"
*/  

                 "<tr><form action='#' method='GET'><td class='col3' >" TXT_DAY_ID ":"                                   // Scene or Pattern to load at day
                "</td><td><input class='col2' name='dayId' value='");   
    message += EEPROM.read(addrDayId);
    message +=F("'></td><td></td><td><input class='col2' type='submit' value='" TXT_SAVE "'></form></tr>\n"   
    
                "</table>\n"
                "</article>\n"); //pgmc
  } // End of Page 2 specifica


  else if (server.uri()=="/3.htm")                                                  //*** 3 channel settings ***
  {
    int channel = 0;
    //first we care for the parameters:
    
    if (server.argName(0)=="CMD" && server.arg(0)=="CC" )
    {
      Serial.println(F("D255 Change Channel"));
      channel = server.arg(1).toInt();
      EEPROM.write(addrChannels+channel*11,server.arg(2).toInt());                        //EEPROM.update will be done with the next custom function
      EEPROMWritestring(addrChannels+1+channel*11,server.arg(server.args()-1));           //memo field ist das letzte Feld
    }

    if (server.argName(0)=="channel")
    {
      Serial.println(F("D181 Change Channel"));
      channel = server.arg(0).toInt();
    }    

    // now the HTML   
    
    message +=F("<article class='channelc'>\n"
              "<h2>" TXT_CHANNEL " " TXT_SETTINGS "</h2>\n"
              );
              
    message+=F("<form action='#' method='GET'>\n<p>" TXT_CHANNEL " ");
    message+=channel;
    message+=F("</p><input type='hidden' name='CMD' value='CC'>\n"      
                 "<input type='hidden' name='channel' value='");          
    message+=channel;
    message+=F("'><input class='col' name='capability' value='");
    message+=EEPROM.read(addrChannels+channel*11);
    message +=F("' id='capability");
    message+=channel;
    message+=F("'>\n");    

    for (int j= 0; j<8; j++)
    {
      message+=F("<input type='checkbox' id='");
      message+=channel;
      message+=F("active");
      message+=j;
      message+=F("' onChange='javascript:updateCapability(");
      message+=channel;
      message+=F(")' name='helper' value='checked' ");
      if (EEPROM.read(addrChannels+channel*11) & (1<<j))
      {
        message+=F("checked='checked'");  
      }
      message+=F(">\n");   
    }
      
    message +=F("<input class='col2' name='memo' value='");      //3
    message+=EEPROMReadstring(addrChannels+1+channel*11);
    message+=F("'>\n"
                 "<input class='col2' type='submit' value='" TXT_SAVE "'>"     
                 "</form>\n");

    message +=F("</article>\n"); //pgmc

  } //end of page 3 specifica


  else if (server.uri()=="/4.htm")                                                  //*** 4 scenes ***
  {
    //first we care for the parameters 
    unsigned int start=0;
    if (server.argName(0)=="start")
    {
      start=server.arg(0).toInt();                     
    }
 
    // now the HTML   
    message+=F("<article class='channelc'>\n<h2>" TXT_SCENES " (");
    message+=sceneCache;
    message+=F(")</h2>\n");
    message+=F("<div id='subm'>");                       // submenu
    byte pages = sceneCache/maxRows;                     // make a small menubar to navigate through the scenes
    for (unsigned int i = 0; i<=pages; i++)              // as the menu is on the top of the page, it only gets updated AFTER a successsfull call of the page
    {
       message+=F(" <a href='?start=");
       message+=i*maxRows;   
       message+=F("'");
       if (start>=i*maxRows && start<((i+1)*maxRows)) message+=F(" class='curr'");    //current menu entry fits to this page
       message+=F(">");
       message+=i*maxRows;
       message+=F("..</a>");
    }
    message+=F("</div>\n");
    message+=F("<table>\n<tr><td>" TXT_NR "</td>"          // 1
               "<td>" TXT_ADDR "</td>"                     // 2
               "<td>" TXT_CHANNELS "</td>\n"               // 3
               "<td>" TXT_MEMO "</td>\n"                   // 4
               "<td>" TXT_CONTENT "</td>\n"                // 5
               "<td></td></tr>\n"                        // 6
               );
    unsigned int sceneAddr = addrScene;
    unsigned i = start;
    byte actualRow = 0;
    while (sceneAddr<addrSequence-11 && actualRow <= maxRows)
    {
      sceneAddr=getSceneAddr(i);
      if (sceneAddr>addrSequence-11) 
         {
          sceneCache = i-1;         // new limit of scenes reached
          break;                    // in seldom cases it could be that the limit is reached after the new getSceneAddr ...
         }
      message+=F("<tr>\n<td class='col'>");
      message+=i;                                 // 1 Nr
      message+=F("</td>\n<td>");
      message+=sceneAddr;                         // 2 Addr
      message +=F("</td>\n<td>");      
      int actualLength=EEPROM.read(sceneAddr);    
      message+=actualLength;                      // 3 Channels
      message+=F("</td>\n<td>");
      message+=EEPROMReadstring(sceneAddr+1);     // 4 Memo  
      message+=F("</td><td>");
//      for (int j=0; j<actualLength;j++)           // 5 Content
//      {
//        message+=EEPROM.read(sceneAddr+1+8+j*2);
//        message+=F(":");
//        message+=EEPROM.read(sceneAddr+1+8+j*2+1);
//        message+=F(" ");
//      }
      message+=F("</td>\n"
                 "<td>"
                 "<button type='button' value='255' id='asceneload' onclick='fire(this.id,");
      message+=i;          
      message+=F(")'>" TXT_LOAD "</button> \n"
                 "<button type='button' value='255' id='ascenesave' onclick='fire(this.id,");
      message+=i;         
      message+=F(")'>" TXT_SAVE "</button> \n"
                 "<a href='5.htm?scene=");                  //MISSING: da brauchen wir ein ändern detaildaten
      message+=i;           
      message+=F("'>" TXT_EDIT "</a></td></tr>\n");
      i++;
      actualRow++;
    }                           
    message +=F("</table>\n</article>\n"); //pgmc

  } //end of page 4 specifica




  else if (server.uri()=="/5.htm")                                                  //*** 5 Scene Settings ***
  {
    int scene = 0;
    //first we care for the parameters:
    
    if (server.argName(0)=="CMD" && server.arg(0)=="CS" )
    {
      Serial.println(F("D412 Change Scene"));
      scene = server.arg(1).toInt();
      unsigned int sceneAddr = getSceneAddr(scene);
      EEPROMWritestring(sceneAddr+1,server.arg(server.args()-1));                   //memo field ist the last field
    }

    if (server.argName(0)=="scene")
    {
      scene = server.arg(0).toInt();
    }    

    // now the HTML   
    
    message +=F("<article class='channelc'>\n"
              "<h2>" TXT_SCENES " - " TXT_SETTINGS "</h2>\n");
              
    message+=F("<form action='#' method='GET'>\n<p>" TXT_SCENE " ");
    message+=scene;
    message+=F("</p><input type='hidden' name='CMD' value='CS'>\n"      
                 "<input type='hidden' name='scene' value='");          
    message+=scene;
    message+=F("'>\n");    

    unsigned int sceneAddr = getSceneAddr(scene);
    message +=F("<input class='col2' name='memo' value='");      //3
    message+=EEPROMReadstring(sceneAddr+1);
    int actualLength=EEPROM.read(sceneAddr); 
    message+=F("'>\n"
                 "<input class='col2' type='submit' value='" TXT_SAVE "'>"     
                 "</form><p>" TXT_ADDRESS ": ");
    message+=sceneAddr;             
    message+=F("</p><p>" TXT_CHANNELS " ");
    message+=actualLength;             
    message+=F(":</p><p>\n");                                            
    for (int j=0; j<actualLength;j++)                            // 5 Content - the saved channels + r/g/b values
      {
        message+=EEPROM.read(sceneAddr+1+8+j*4);
        message+=F(":");
        message+=EEPROM.read(sceneAddr+1+8+j*4+1);
        message+=F("/");
        message+=EEPROM.read(sceneAddr+1+8+j*4+2);
        message+=F("/");
        message+=EEPROM.read(sceneAddr+1+8+j*4+3);
        message+=F(" ");
      }

    message +=F("</p></article>\n"); //pgmc

  } //end of page 5 specifica


  message +="</main>\n<footer>\n<p>";
  if (ss>604800)
            {
              message +="<span id='week'>";
              message +=  ((ss/604800)%52);
              message +=  "</span> " TXT_WEEKS " ";
            }
  if (ss>86400)
            {
              message +="<span id='day'>";
              message += ((ss/86400)%7);
              message +=  "</span> " TXT_DAYS " ";
            }
  if (ss>3600)
            {
              message +="<span id='hour'>";
              message += ((ss/3600)%24);
              message += "</span> " TXT_HOURS " ";
            }
  if (ss>60)
            {
              message +=F("<span id='min'>");
              message +=  ((ss/60)%60);
              message +=F("</span> " TXT_MINUTES " ");
            }
  message +=F("<span id='sec'>");
  message +=  (ss%60); 
  message +=F("</span> " TXT_SECONDS " | " TXT_VERSION " " VERSION " </p>\n</footer>"    
              "</body>\n</html>");
  server.send(200, "text/html", message);
  digitalWrite(LED_RED_PIN, 0);
}


void handleCss()
  //.valuecontainer = .valc    .programcontainer = .pgmc
  //full CSS is Flash to spare about 400 byte PROGMEM and 270 byte RAM!
{
  digitalWrite(LED_RED_PIN, 1);
  Serial.println(F("D272 requested f.css"));
  String message;
  message =F("*{font-family:sans-serif}\n" 
             "body{margin:10px}\n"
             "h1, h2{color:white;background:#8A0829;text-align:center}\n"
             "h1{font-size:1.2em;margin:1px;padding:5px}\n"
             "h2{font-size:1.0em}\n"
             "h3{font-size:0.9em}\n"
             "a{text-decoration:none;color:dimgray;text-align:center}\n"
             "main{text-align:center}\n"
             "article{vertical-align:top;display:inline-block;margin:0.2em;padding:0.1em;border-style:solid;border-color:#C0C0C0;text-align:center;background-color:#E5E5E5;width:15em}\n"
             "article h2{margin-top:0;padding-bottom:1px}\n"
             ".pgmc{text-align:left;width:20em}\n"
             ".channelc{text-align:left;width:27em}\n"
             "section {margin-bottom:0.2em;clear:both;}\n"
             "table {border-collapse:separate;border-spacing:0 0.2em}\n"
             "td{background-color:#C0C0C0;font-size:0.7em}\n"
             ".bgc1{background-color:white}\n"
             ".bgc2{background-color:red}\n"
             ".bgc3{background-color:green}\n"
             ".bgc4{background-color:blue}\n"
             "button{margin-top:0.3em}\n"
             ".small{font-size:0.6em}\n"
             ".value{font-size:1.8em;text-align:center;line-height:50%}\n"
             "footer p, .infodaten p{font-size:0.7em;color:dimgray;background:silver;text-align:center;margin-bottom:5px}\n"
             "nav{background-color:silver;margin:1px;padding:5px;font-size:0.8em}\n"
             "nav a{color:white;padding:10px;text-decoration:none}\n"
             "nav a:hover{text-decoration:underline}\n"
             "nav p{margin:0px;padding:0px}\n"
             ".on, .off{margin-top:0;margin-bottom:0.2em;margin-left:3em;font-size:1.4em;background-color:#C0C0C0;border-style:solid;width:5em;height:1.5em;text-decoration:none;text-align:center}\n"
             ".on{border-color:green}\n"
             ".off{border-color:red}\n"
             ".functionOn{background-color:green}\n" 
             ".col, .col2, .col3{margin-right:0.1em;width:2em}\n"
             ".col2{width:6em}\n"
             ".col3{width:15em}\n"
             ".col4{width:18em}\n"
             ".group{background-color:#C0C0C0;display:inline-block;margin-bottom:0.1em;height:1.8em}\n"
             "#subm {margin-bottom:0.2em;}\n"
             "#subm a{color:#606060;font-size:0.8em;padding: 0.2em 0.6em;background:silver;border:1px solid darkgrey;border-radius: 0.5em 0.5em 0em 0em;-moz-border-radius:0.5em 0.5em 0em 0em;}\n"
             "#subm a:hover{background:#f0f0f0;}\n"
             "#subm a.curr{background:#D9D9D9;}");
  server.send(200, "text/css", message);
  digitalWrite(LED_RED_PIN, 0);
}


void handleAjax() {
  digitalWrite(LED_RED_PIN, 1);
  Serial.println(F("D626 requested .js"));
  String message ="";
  message+= millis()/1000;
  message+= ",";
  if (server.argName(0)=="p" && (server.arg(0)=="/0.htm.js" || server.arg(0)=="/.js"))      //request with p=0  p page ... 0 0.htm
  {
    for (int j = 0; j<maxChannels; j++)              
    { 
        message+= r_channel[j];
        message+= ",";  
        message+= g_channel[j];
        message+= ",";
        message+= b_channel[j];
        message+= ",";        
    }
    message+= master;
    message+= ",";
    message+= isBlackout;
    message+= ",";
  }
  if (server.argName(0)=="w")                                                    // white slider was moved
  {
    unsigned int actualChannel=server.arg(0).toInt();
    //Serial.print(F("D624 ")); Serial.print (server.argName(0)); Serial.print(F(" ")); Serial.println(actualChannel);
    //for white wikipedia recommends to use 50% red, 100% green, 30% blue, but 95/100/90 looks better and warmer white for my LED stripes.
    g_channel[actualChannel]=constrain(server.arg(1).toInt(),0,255);             // green=100% is the reference. Green is actualChannel+1
    r_channel[actualChannel]=constrain(g_channel[actualChannel]*95/100,0,255);   // red
    b_channel[actualChannel]=constrain(g_channel[actualChannel]*90/100,0,255);   // blue
    updatePixel();
  } 
  else if (server.argName(0)=="a")                                                // action button was pressed
  {
    if (server.arg(0)=="scenesave")                                               // Button sceneload
    {
       if (server.argName(1)=="v")
       {
         Serial.println(F("D459 Scene Save"));
         saveScene(server.arg(1).toInt());
       }
    }
    else if (server.arg(0)=="sceneload")                                          // Button sceneload
    {
       if (server.argName(1)=="v")
       {
         Serial.println(F("D513 Scene Load"));
         loadScene(server.arg(1).toInt());
         actualPattern=0;                                                         // on scene load we switch of any other running pattern
       }
    }
    else if (server.arg(0)=="fade")                                               // Button Fade
    {
       Serial.print (F("D689 Button fade: "));
       if (actualPattern==1) 
       {actualPattern=0;} 
       else 
       {
         for (int j = 0; j<maxChannels; j++)                                      // Randomize before fade
         {
           r_channel[j] = random(0,255);
           g_channel[j] = random(0,255);
           b_channel[j] = random(0,63);  //blue is very intensive ... limited to lower value
         }
         actualPattern=1;
       }           
       // Pattern on or off
       Serial.println(actualPattern);
    }
    else if (server.arg(0)=="pattern2")                                           // Button Pattern2
    {
       Serial.print (F("D692 Button Pattern2: "));
       if (actualPattern==2) {actualPattern=0;} else {actualPattern=2; counterPattern=0;}           // Pattern on or off
       Serial.println(actualPattern);
       
    }
    else if (server.arg(0)=="pattern3")                                           // Button Pattern
    {
       Serial.print (F("D701 Button Pattern3: "));
       if (actualPattern==3) {actualPattern=0;} else {actualPattern=3; counterPattern=0;}           
       Serial.println(actualPattern);
    }
    else if (server.arg(0)=="pattern4")                                           // Button Pattern
    {
       Serial.print (F("D707 Button Pattern4: "));
       if (actualPattern==4) {actualPattern=0;} else {actualPattern=4;counterPattern=0;}           
       Serial.println(actualPattern);
    }
    else if (server.arg(0)=="pattern5")                                           // Button Pattern
    {
       Serial.print (F("D713 Button Pattern5: "));
       if (actualPattern==5) {actualPattern=0;} else {actualPattern=5;counterPattern=0;}           
       Serial.println(actualPattern);
    }    
    else if (server.arg(0)=="pattern6")                                           // Button Pattern
    {
       Serial.print (F("D793 Button Pattern6: "));
       if (actualPattern==6) {actualPattern=0;} else {actualPattern=6;counterPattern=0;}           
       Serial.println(actualPattern);
    }
    else if (server.arg(0)=="pattern7")                                           // Button Pattern
    {
       Serial.print (F("D799 Button Pattern7: "));
       if (actualPattern==7) {actualPattern=0;} else {actualPattern=7;counterPattern=0;}           
       Serial.println(actualPattern);
    }
    else if (server.arg(0)=="rnd")                                                // Button Random
    {
      for (int j = 0; j<maxChannels; j++) 
      {
        r_channel[j] = random(0,255);
        g_channel[j] = random(0,255);
        b_channel[j] = random(0,63);  //blue is very intensive ... limited to lower value
      }
      actualPattern=0;
      updatePixel();
    } 
    else if (server.arg(0)=="del")                                                 // Button Delete
    {
      //for (int j = 0; j<maxChannels; j++) {r_channel[j] = 0;g_channel[j] = 0;b_channel[j] = 0;}
      memset(r_channel,0,maxChannels);
      memset(g_channel,0,maxChannels);
      memset(b_channel,0,maxChannels);
      actualPattern=0;
      updatePixel();
    } 
    else if (server.arg(0)=="black")                                               //Button Blackout
    {
      if (isBlackout) {isBlackout=false;} else {isBlackout=true;}
      updatePixel();
    } 
  }
  if (server.argName(0)=="r" && server.arg(0).toInt()<=maxChannels)                // handle Slider changes - MISSING - hat eigentlich nichts im AJAX verloren und gehört auf eine Separate Seite
  {
    if (server.argName(1)=="v")
    {
      r_channel[server.arg(0).toInt()]= constrain(server.arg(1).toInt(),0,255);
      updatePixel();  
    }
  }
  else if (server.argName(0)=="g" && server.arg(0).toInt()<=maxChannels)                
  {
    if (server.argName(1)=="v")
    {
      g_channel[server.arg(0).toInt()]= constrain(server.arg(1).toInt(),0,255);
      updatePixel();  
    }
  }
  else if (server.argName(0)=="b" && server.arg(0).toInt()<=maxChannels)                
  {
    if (server.argName(1)=="v")
    {
      b_channel[server.arg(0).toInt()]= constrain(server.arg(1).toInt(),0,255);
      updatePixel();  
    }
  }

  
  if (server.argName(0)=="c" && server.arg(0)=="master")                           // Master Volume changed
  {
    //Serial.print(F("D448 Master")); Serial.println(master);
    if (server.argName(1)=="v")
    {
      //Serial.print(F("D452 Master")); Serial.println(master);
      master=constrain(server.arg(1).toInt(),0,100);
      updatePixel();
    }
  }
 
  server.send(200, "text/plain", message);
  digitalWrite(LED_RED_PIN, 0);
}


void handleJs() {
  digitalWrite(LED_RED_PIN, 1);
  String message;

if (server.uri()=="/3.htm.js")                       //helperfunction for channel change page - updates the capability field according to the checkboxes.
{
  message +=F(
  "function updateCapability(channel){\n"
  "  var c=0;\n"
  "  if (document.getElementById(channel+'active0').checked) {c+=1;}\n"
  "  if (document.getElementById(channel+'active1').checked) {c+=2;}\n"
  "  if (document.getElementById(channel+'active2').checked) {c+=4;}\n"
  "  if (document.getElementById(channel+'active3').checked) {c+=8;}\n"
  "  if (document.getElementById(channel+'active4').checked) {c+=16;}\n"
  "  if (document.getElementById(channel+'active5').checked) {c+=32;}\n"
  "  if (document.getElementById(channel+'active6').checked) {c+=64;}\n"
  "  if (document.getElementById(channel+'active7').checked) {c+=128;}\n"
  "  document.getElementById('capability'+channel).value=c;\n"
  "}\n");
}

if (server.uri()=="/0.htm.js" || server.uri()=="/.js" || server.uri()=="/4.htm.js")  //homepage could be called without any page ...
{
  message +=F(  
  "  function fire(c,v){\n"                                   // c channel (mostly the id/self) | v myValue
  "    var p = 'c';\n"                                        // the default parameter = c (channel)
  "    if (c.charAt(0)=='w') {p='w'; c=c.substr(1);}\n"       // parameter p = w (white slider)
  "    if (c.charAt(0)=='r') {p='r'; c=c.substr(1);}\n"       // parameter p = r (red slider)
  "    if (c.charAt(0)=='g') {p='g'; c=c.substr(1);}\n"       // parameter p = g (green slider)
  "    if (c.charAt(0)=='b') {p='b'; c=c.substr(1);}\n"       // parameter p = b (blue slider)    
  "    if (c.charAt(0)=='a') {p='a'; c=c.substr(1);}\n"       // parameter p = a (action button)
  "    var request = new XMLHttpRequest(); \n"
  "    request.open('GET', 'ajax?'+p+'='+c+'&v='+v, true);\n"
  "    request.send(null);\n"
  "    if (p=='w'||c=='black'||c=='rnd'||c=='del'||c=='sceneload'){GetSwitchState(1);}\n"   // after some actions the ajax should be called to update the page
  "  }\n");
}
message +=F(    
"  function GetSwitchState(s) {\n"                            // 0 call function with settimeout, 1 run only once for manual update
"   nocache = '&nocache='+ Math.random() * 1000000;\n"
"   var request = new XMLHttpRequest(); \n"
"   request.onreadystatechange = function() {\n"
"   if (this.readyState == 4) {\n"
"    if (this.status == 200) {\n"
"     if (this.responseText != null) {\n"
"      var value=this.responseText.split(',')\n"
"      document.getElementById('sec').innerHTML = Math.floor(value[0]%60);\n"
"      if (value[0]>60){document.getElementById('min').innerHTML = Math.floor(value[0]/60%60);}\n"
"      if (value[0]>3600){document.getElementById('hour').innerHTML = Math.floor(value[0]/3600%24);}\n"
"      if (value[0]>86400){document.getElementById('day').innerHTML = Math.floor(value[0]/86400%7);}\n"
"      if (value[0]>604800){document.getElementById('week').innerHTML = Math.floor(value[0]/604800%52);}\n");

if (server.uri()=="/0.htm.js" || server.uri()=="/.js")  //homepage could be called without any page ...
{
   message+=F("      for (var j = 0; j<");                                                                       // The channels
   message+=maxChannels;
   message+=F("; j++){\n"
              "       if (document.getElementById('r'+j)) {document.getElementById('r'+j).value = value[j*3+1];}\n"
              "       if (document.getElementById('g'+j)) {document.getElementById('g'+j).value = value[j*3+2];}\n"
              "       if (document.getElementById('b'+j)) {document.getElementById('b'+j).value = value[j*3+3];}}\n"
   
              "      document.getElementById('master').value = value[");                                          // The Master Volume
   message+=maxChannels*3+1;
   message+=F("];\n" 

              "      if (document.getElementById('ablack')) {if (value[");                                        // Blackout Button
   message+=maxChannels*3+2;
   //message+=F("]==1){document.getElementById('ablack').style.backgroundColor ='#00FF00';} else {document.getElementById('ablack').style.backgroundColor = '';} };\n");
   message+=F("]==1){document.getElementById('ablack').className ='functionOn';} else {document.getElementById('ablack').className ='';} };\n");
}

message+=F(
"      document.getElementById('sec').style.color = 'dimgray';\n"
"   }}}\n"
"   else {document.getElementById('sec').style.color = 'red';}\n"
"   }\n"
"   request.open('GET', 'ajax?p=");
message+=server.uri();
message+=F("' + nocache, true);\n"
"   request.send(null);\n"
"   if (s==0) setTimeout('GetSwitchState(0)', ");
message+=ajaxIntervall*1000;
message+=F(
");\n"
"   }");

  server.send(200, "text/plain", message);
  digitalWrite(LED_RED_PIN, 0);
}

